import { IObservableArray } from "mobx";
import { IAnyType, IStateTreeNode, IType, OptionalProperty, ExtractS, ExtractC, ExtractT, ExtractCST } from "../../internal";
/** @hidden */
export interface IMSTArray<IT extends IAnyType> extends IObservableArray<ExtractT<IT>>, IStateTreeNode<ExtractC<IT>[] | undefined, ExtractS<IT>[]> {
    push(...items: ExtractT<IT>[]): number;
    push(...items: ExtractCST<IT>[]): number;
    concat(...items: ConcatArray<ExtractT<IT>>[]): ExtractT<IT>[];
    concat(...items: ConcatArray<ExtractCST<IT>>[]): ExtractT<IT>[];
    concat(...items: (ExtractT<IT> | ConcatArray<ExtractT<IT>>)[]): ExtractT<IT>[];
    concat(...items: (ExtractCST<IT> | ConcatArray<ExtractCST<IT>>)[]): ExtractT<IT>[];
    splice(start: number, deleteCount?: number): ExtractT<IT>[];
    splice(start: number, deleteCount: number, ...items: ExtractT<IT>[]): ExtractT<IT>[];
    splice(start: number, deleteCount: number, ...items: ExtractCST<IT>[]): ExtractT<IT>[];
    unshift(...items: ExtractT<IT>[]): number;
    unshift(...items: ExtractCST<IT>[]): number;
}
/** @hidden */
export interface IArrayType<IT extends IAnyType> extends IType<ExtractC<IT>[] | undefined, ExtractS<IT>[], IMSTArray<IT>>, OptionalProperty {
}
/**
 * `types.array` - Creates an index based collection type who's children are all of a uniform declared type.
 *
 * This type will always produce [observable arrays](https://mobx.js.org/refguide/array.html)
 *
 * Example:
 * ```ts
 * const Todo = types.model({
 *   task: types.string
 * })
 *
 * const TodoStore = types.model({
 *   todos: types.array(Todo)
 * })
 *
 * const s = TodoStore.create({ todos: [] })
 * unprotect(s) // needed to allow modifying outside of an action
 * s.todos.push({ task: "Grab coffee" })
 * console.log(s.todos[0]) // prints: "Grab coffee"
 * ```
 *
 * @param subtype
 * @returns
 */
export declare function array<IT extends IAnyType>(subtype: IT): IArrayType<IT>;
/**
 * Returns if a given value represents an array type.
 *
 * @param type
 * @returns `true` if the type is an array type.
 */
export declare function isArrayType<Items extends IAnyType = IAnyType>(type: IAnyType): type is IArrayType<Items>;
